/*****************************************************************************/
/*
                                   Net.h
*/
/*****************************************************************************/

#ifndef NET_H_LOADED
#define NET_H_LOADED 1

#include "wasd.h"

#define DEFAULT_HTTP_PORT          80
#define DEFAULT_HTTPS_PORT        443
#define DEFAULT_HTTP_PROXY_PORT  8080
#define DEFAULT_CONNECT_PORT     4443
#define DEFAULT_LISTEN_BACKLOG      5

/* output character encoding */
#define NET_WRITE_BUFFERED_NONE     0
#define NET_WRITE_BUFFERED_HTMLESC  1
#define NET_WRITE_BUFFERED_URLENC   2

/**************/
/* structures */
/**************/

struct ServiceStruct
{
   struct ServiceStruct  *NextPtr;

   boolean  ConnectService,
            LogFileOpen,
            LogFileError,
            LocalAuthorizationRequired,
            ProxyAuthorizationRequired,
            ProxyFileCacheEnabled,
            ProxyService,
            TrackEnabled;

   int  ClientPort,
        ConnectCount,
        ListenBacklog,
        LogFileNameLength,
        ProxyChainHostNameLength,
        ProxyChainPort,
        RequestScheme,
        ServerHostNameLength,
        ServerPort;

   unsigned long  ClientIpAddress,
                  ProxyChainIpAddress,
                  ServerHostNameHashValue,
                  ServerHostPortHashValue,
                  ServerIpAddress;

   unsigned long  QuadBytesRawRx [2],
                  QuadBytesRawTx [2];

   char  *RequestSchemeNamePtr;

   char  ClientHostName [128],
         ClientIpAddressString [32],
         LogFileName [256],
         LogHostName [64],
         ProxyChainHostName [128],
         ProxyChainHostPort [128+16],
         ProxyChainIpAddressString [32],
         ProxyChainPortString [16],
         ServerHostName [128],
         ServerHostPort [128+16],
         ServerIpAddressString [32],
         ServerPortString [16],
         SeSoLaCert [128],
         SeSoLaCipherList [256],
         SeSoLaKey [128],
         SuppliedIpAddressString [32];

   struct AnIOsb  ServerIOsb;

   struct dsc$descriptor  LogFileNameDsc;
   
   struct FAB  LogFileFab;
   struct RAB  LogFileRab;

   /* this will be cast using (SSL_CTX*) in SESOLA.C */
   void  *SeSoLaCtx;

   unsigned short  ClientChannel,
                   ServerChannel;

   struct sockaddr_in  ServerSocketName,
                       ClientSocketName;

   struct {
      unsigned int  Length;
      char  *Address;
   } ServerSocketNameItem;

   int  ClientSocketNameLength;
   struct {
      unsigned int  Length;
      char  *Address;
      unsigned int  *LengthPtr;
   } ClientSocketNameItem;
};

/***********************/
/* function prototypes */
/***********************/

NetAccept (struct ServiceStruct*);
NetAcceptAst (struct ServiceStruct*);
NetAccessDenied (unsigned short, char*, char*);
NetBreakSocket (struct RequestStruct*);
NetBreakSocketAst (struct RequestStruct*);
NetClientAddressLookup (struct ServiceStruct*);
NetCreateService ();
NetWriteDirect (unsigned short, char*, int);
NetDummyRead (unsigned short);
NetDummyReadAst (unsigned short);
NetGetServerHostName ();
NetHostNameLookup (char*, int, char*, char*, char*, char*, int*);
NetParseHostString (char*, char*, int, int*);
unsigned long NetHostNameHash (char*);
unsigned long NetHostPortHash (char*, char*);
NetListen ();
NetResponseHeaderAst (struct RequestStruct*);
NetServerShutdown (unsigned short);
char* NetServiceNextHostPort (unsigned long*);
int NetServiceReportConfig (struct RequestStruct*, struct ConfigStruct*);
int NetServiceReportStats (struct RequestStruct*);
NetServiceZeroAccounting ();
NetTooBusy (unsigned short, char*, char*);
boolean NetServiceConfigured (char*);
NetSetService ();
NetCloseSocket (struct RequestStruct*);
int NetRead (struct RequestStruct*, void*, char*, int);
int NetReadRaw (struct RequestStruct*, void*, char*, int);
NetReadRawAst (struct RequestStruct*);
boolean NetVirtualService (struct RequestStruct*);
boolean NetThisVirtualService (char*, char*);

int NetWrite (struct RequestStruct*, void*, char*, int);
int NetWriteBuffered (struct RequestStruct*, void*, char*, int);
int NetWriteRaw (struct RequestStruct*, void*, char*, int);
NetWriteRawAst (struct RequestStruct*);

/* vector print to network buffer */
#define NetWriteFaol(rq,fs,vp) WriteFormatted(rq,NULL,0,NULL,fs,vp)
/* for naming consistency */
#define NetWriteFao WriteFaoNet

/* definition to flush the buffer only if there is more than a buffer-full */
#define NetWritePartFlush(rq,ast) NetWriteBuffered(rq,ast,NULL,-1)

/* to flush buffered contents completely */
#define NetWriteFullFlush(rq,ast) NetWriteBuffered(rq,ast,NULL,0)

/* initialize the buffer */
#define NetWriteInit(rq) NetWriteBuffered(rq,NULL,NULL,0)

#endif /* NET_H_LOADED */

/*****************************************************************************/

                                 
